local name, private = ...

private.objects.storage = private.objects.base()

local storage = private.objects.storage

local allowedKeys = private.objects.list({
    "number",
    "string",
})

local allowedValues = private.objects.list({
    "number",
    "string",
    "boolean",
    "table",
})

local function copyDataTable(tbl)
    -- Copies all data in tbl into a new table and returns the new table.
    -- Discards all keys and values not in the lists above
    -- Does NOT handle recursion!
    if not allowedValues:has(type(tbl)) then return end
    if type(tbl) ~= "table" then return tbl end
    local r = {}
    for k, v in pairs(tbl) do
        if allowedKeys:has(type(k)) and allowedValues:has(type(v)) then
            r[k] = copyDataTable(v)
        end
    end
    return r
end

function storage:init(dblist)
    if type(dblist) ~= "table" then dblist = { dblist } end -- In case only one db name gets passed
    self.addonname = name
    self.dblist = dblist
    self:RegisterEvent("ADDON_LOADED")
    self:RegisterEvent("PLAYER_LOGOUT")
    local savedata = {}
    
    -- Check if the requested data has already been loaded
    for k, v in pairs(self.dblist) do
        if _G[v] then savedata[v] = copyDataTable(_G[v]) end
    end
    
    function self:event(event, name)
        if name ~= self.addonname then return end
        if event == "ADDON_LOADED" then
            for k, v in pairs(self.dblist) do
                savedata[v] = copyDataTable(_G[v]) -- Make a copy, so it cannot be modified except through these functions
            end
        end
        if event == "PLAYER_LOGOUT" then
            for k, v in pairs(self.dblist) do
                _G[v] = savedata[v] -- No need to copy here; this goes into the globals table anyway
            end
        end
    end
    
    function self:Load(dbname)
        -- Loads saved data
        -- The data returned by this function will be a copy of the
        -- saved data. Modifying it will not automatically save the
        -- changes.
        if not savedata[dbname] then return end
        return copyDataTable(savedata[dbname])
    end
    
    function self:Prepare(dbname, data)
        -- Prepares data to save
        -- Note: Data is only actually saved when the UI unloads.
        -- This will happen automatically when the game is shut down,
        -- when the player logs out, or when the UI is reloaded.
        -- It will NOT happen if the game crashes.
        -- Use storage:Save() to immediately save all prepared data.
        -- This will discard all unprepared data.
        
        -- Note: The data passed will be copied. All table keys must
        -- be either strings or numbers. All table values must be
        -- either strings, numbers, booleans, or tables. Further
        -- modification to the passed table after preparing will NOT
        -- change the data that will be saved; it must be prepared
        -- again.
        if not savedata[dbname] then return end
        savedata[dbname] = copyDataTable(data)
    end
    
    function self:Save()
        -- Reloads the UI, immediately saving all data
        -- ONLY use after calling Save() on ALL data
        ReloadUI()
    end
end