local name, private = ...

local list = private.objects.list

private.objects.ui = {}
private.objects.ui.base = {}

local L = private.L

------------------------------------------------------------
-------------------------- Label ---------------------------
------------------------------------------------------------

private.objects.ui.label = private.objects.base()

function private.objects.ui.label:init(parent, text, size)
    self.frame = parent:CreateFontString()
    self.frame:SetFont("Fonts\\FRIZQT__.TTF", size or 12)
    self.frame:SetText(text)
end

function private.objects.ui.label:SetText(text)
    self.frame:SetText(text)
end

------------------------------------------------------------
----------------------- Count label ------------------------
------------------------------------------------------------

private.objects.ui.countlabel = private.objects.base()

function private.objects.ui.countlabel:init(parent, size)
    self.frame = parent:CreateFontString()
    self.frame:SetFont("Fonts\\FRIZQT__.TTF", size or 12)
    self.frame:SetText("0 / 0")
    
    self.left = 0
    self.right = 0
end

function private.objects.ui.countlabel:SetLeft(text)
    self.left = text
    self.frame:SetText(self.left .. " / " .. self.right)
end

function private.objects.ui.countlabel:SetRight(text)
    self.right = text
    self.frame:SetText(self.left .. " / " .. self.right)
end

function private.objects.ui.countlabel:SetBoth(left, right)
    self.left = left
    self.right = right
    self.frame:SetText(self.left .. " / " .. self.right)
end

------------------------------------------------------------
--------------------------- Bar ----------------------------
------------------------------------------------------------

private.objects.ui.bar = private.objects.base()

local green = { r = 0, g = 1, b = 0 }
local yellow = { r = 1, g = 1, b = 0 }

function private.objects.ui.bar:init(colour, parent, overfillcolour)
    self.bgframe = CreateFrame("Frame", nil, parent or UIParent)
    self.fillframe = CreateFrame("Frame", nil, self.bgframe)
    self.overfillframe = CreateFrame("Frame", nil, self.bgframe)
    
    -- Base measures
    self.bgframe:SetSize(80, 10)
    self.fillframe:SetPoint("LEFT")
    self.fillframe:SetSize(40, 10)
    self.fillframe:Hide()
    self.overfillframe:SetPoint("LEFT", self.fillframe, "RIGHT")
    self.overfillframe:SetSize(40, 10)
    self.overfillframe:Hide()
    self.length = 0
    self.overlength = 0
    
    self:SetColour(colour, overfillcolour)
end

function private.objects.ui.bar:SetColour(colour, overfillcolour)
    -- Takes a table holding keys r, g, b and a.
    
    -- Use green for overfill if nothing else was specified
    if not colour then colour = green end
    
    -- Use yellow for overfill if nothing else was specified
    if not overfillcolour then overfillcolour = yellow end
    
    -- Set the background's transparency first
    local t = self.bgframe.texture or self.bgframe:CreateTexture()
    t:SetTexture(0.3, 0.3, 0.3, colour.a or 1)
    t:SetAllPoints(self.bgframe)
    self.bgframe.texture = t
    
    -- Then set the fill bar's colour and transparency
    t = self.fillframe.texture or self.fillframe:CreateTexture()
    t:SetTexture(colour.r, colour.g, colour.b, colour.a or 1)
    t:SetAllPoints(self.fillframe)
    self.fillframe.texture = t
    
    -- Finally, the overfill bar
    t = self.overfillframe.texture or self.overfillframe:CreateTexture()
    t:SetTexture(overfillcolour.r, overfillcolour.g, overfillcolour.b, overfillcolour.a or 1)
    t:SetAllPoints(self.overfillframe)
    self.overfillframe.texture = t
end

function private.objects.ui.bar:SetSize(width, height)
    -- Sets the width and height of both frames
    self.bgframe:SetSize(width, height)
    self.fillframe:SetWidth(width)
    self:SetFill(self.length)
    self:SetOverfill(self.overlength)
end

function private.objects.ui.bar:SetFill(length)
    -- Takes a number between 0 and 1
    -- Fills the bar accordingly
    
    -- Numbers outside of that range will be read as either 0 or 1
    length = max(length, 0)
    length = min(length, 1)
    
    -- Calculate the shown overfill
    local overfill = min(self.overlength, 1 - length)
    
    if length > 0 then
        self.fillframe:Show()
        self.fillframe:SetWidth(length * self.bgframe:GetWidth())
    else
        self.fillframe:Hide()
    end
    
    if overfill > 0 then
        self.overfillframe:Show()
        self.overfillframe:SetWidth(overfill * self.bgframe:GetWidth())
    else
        self.overfillframe:Hide()
    end
    
    -- Record the length in case the bar's size changes
    self.length = length
end

function private.objects.ui.bar:SetOverfill(overlength)
    -- Takes a number between 0 and 1
    -- Will show the overfill bar based on that number and the length
    -- of the regular fill
    
    -- Numbers outside of that range will be read as either 0 or 1
    length = max(overlength, 0)
    length = min(overlength, 1)
    
    -- Calculate the shown overfill
    local overfill = min(overlength, 1 - self.length)
    
    if overfill > 0 then
        self.overfillframe:Show()
        self.overfillframe:SetWidth(overfill * self.bgframe:GetWidth())
    else
        self.overfillframe:Hide()
    end
    
    self.overlength = overlength
end

function private.objects.ui.bar:SetPoint(...)
    self.bgframe:SetPoint(...)
end

------------------------------------------------------------
------------------------- Race bar -------------------------
------------------------------------------------------------

private.objects.ui.raceBar = private.objects.base()

local function raceBarClickHandler(self, button)
    if button == "LeftButton" then
        -- NYI - populate scroll frame
    else
        -- Solve the artifact
        self.__parent.raceObject:Solve()
    end
end

function private.objects.ui.raceBar:init(raceID, parent)
    self.frame = CreateFrame("Button", nil, parent or UIParent)
    self.frame:SetSize(520, 20)
    self.frame.__parent = self
    self.frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    self.frame:SetScript("OnEnter", self.ShowTooltip)
    self.frame:SetScript("OnLeave", self.HideTooltip)
    self.frame:SetScript("OnClick", raceBarClickHandler)
    
    self.raceObject = private.objects.data.race:new()
    self.raceObject:init(raceID)
    
    self.raceName = private.objects.ui.label()
    self.raceName:init(self.frame, self.raceObject.name)
    self.raceName.frame:SetPoint("LEFT", 0, 1)
    
    self.overall = private.objects.ui.bar()
    self.overall:init(green, self.frame)
    self.overall.bgframe:SetPoint("LEFT", 80, 0)
    
    self.overallCount = private.objects.ui.countlabel()
    self.overallCount:init(self.frame)
    self.overallCount.frame:SetPoint("LEFT", 170, 1)
    
    self.current = private.objects.ui.bar()
    self.current:init(green, self.frame)
    self.current.bgframe:SetPoint("LEFT", 250, 0)
    
    self.currentCount = private.objects.ui.countlabel()
    self.currentCount:init(self.frame)
    self.currentCount.frame:SetPoint("LEFT", 340, 1)
    
    self.keystones = private.objects.ui.countlabel()
    self.keystones:init(self.frame)
    self.keystones.frame:SetPoint("RIGHT", 0, 1)
end

function private.objects.ui.raceBar:update()
    self.raceObject:update()
    
    self.overall:SetFill(self.raceObject.completed.total / self.raceObject.numArtifacts.total)
    self.overallCount:SetBoth(self.raceObject.completed.total, self.raceObject.numArtifacts.total)
    --print(self.raceObject.completed.total, self.raceObject.numArtifacts.total)
    
    self.current:SetFill(self.raceObject.regularFragments / self.raceObject.currentArtifact.fragments)
    self.current:SetOverfill(self.raceObject.keystoneFragments / self.raceObject.currentArtifact.fragments)
    self.currentCount:SetBoth(self.raceObject.totalFragments, self.raceObject.currentArtifact.fragments)
    
    if self.raceObject.keystoneID == 0 then
        self.keystones.frame:Hide()
    else
        self.keystones:SetBoth(self.raceObject.numKeystones, self.raceObject.currentArtifact.sockets)
    end
end

function private.objects.ui.raceBar:ShowTooltip()
    GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
    GameTooltip:ClearLines()
    GameTooltip:AddLine(self.__parent.raceObject.name)
    GameTooltip:AddLine(format(
        L("%d / %d artifacts completed."),
        self.__parent.raceObject.completed.total,
        self.__parent.raceObject.numArtifacts.total
    ))
    GameTooltip:AddLine(format(
        L("%d solves total."),
        self.__parent.raceObject.totalCompleted
    ))
    GameTooltip:AddLine(" ")
    GameTooltip:AddDoubleLine(L("Project:"), self.__parent.raceObject.currentArtifact.name, 1, 1, 0, 1, 1, 1)
    local ready = (self.__parent.raceObject.totalFragments >= self.__parent.raceObject.currentArtifact.fragments)
    GameTooltip:AddDoubleLine(
        format(
            L("%d / %d fragments"),
            self.__parent.raceObject.totalFragments,
            self.__parent.raceObject.currentArtifact.fragments
        ),
        ready and
            L("Ready to solve!")
        or
            format(
                L("Need %d more."),
                self.__parent.raceObject.currentArtifact.fragments - self.__parent.raceObject.totalFragments
            ),
        1, 1, 1, ready and 0 or 1, ready and 1 or 0, 0 -- Yep, this is ugly.
    )
    GameTooltip:AddLine(" ")
    GameTooltip:AddLine(L("Click to open artifact list. [NYI]"), 0, 1, 0)
    if ready then GameTooltip:AddLine(L("Right click to solve."), 0, 1, 0) end
    
    GameTooltip:Show()
end

function private.objects.ui.raceBar:HideTooltip()
    GameTooltip:Hide()
end

------------------------------------------------------------
------------------------ Base frame ------------------------
------------------------------------------------------------

private.objects.ui.base.frame = private.objects.base()

local frameBackdrop = {
    bgFile = "Interface\\TutorialFrame\\TutorialFrameBackground",
    edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
    tile = true, tileSize = 16, edgeSize = 16,
    insets = { left = 5, right = 5, top = 5, bottom = 5 } }

function private.objects.ui.base.frame:init(parent)
    self.frame = CreateFrame("Frame", nil, parent or UIParent)
    self.frame:SetBackdrop(frameBackdrop)
end

------------------------------------------------------------
------------------------ Race frame ------------------------
------------------------------------------------------------

local continents = {
    { 1, 3, 4, 7, 8 }, -- Kalimdor
    { 1, 3, 4, 5, 8 }, -- Eastern Kingdoms
    { 2, 6 }, -- Outland
    { 4, 5, 8, 9 }, -- Northrend
}

private.objects.ui.raceFrame = private.objects.ui.base.frame()

function private.objects.ui.raceFrame:init(parent)
    -- Set up the frame
    self.frame = CreateFrame("Frame", nil, parent or UIParent)
    self.frame:SetBackdrop(frameBackdrop)
    
    self.frame:EnableMouse(true)
    self.frame:SetMovable(true)
    self.frame:SetScript("OnDragStart", self.frame.StartMoving)
    self.frame:SetScript("OnDragStop", self.frame.StopMovingOrSizing)
    self.frame:RegisterForDrag("LeftButton")
    self.frame:SetWidth(550)
    self.frame:SetPoint("CENTER")
    
    self.raceLabel = private.objects.ui.label()
    self.raceLabel:init(self.frame, "|c00FFFF00" .. L("Race") .. "|r", 16)
    self.raceLabel.frame:SetPoint("TOPLEFT", 15, -12)
    
    self.progressLabel = private.objects.ui.label()
    self.progressLabel:init(self.frame, "|c00FFFF00" .. L("Progress") .. "|r", 16)
    self.progressLabel.frame:SetPoint("TOPLEFT", 95, -12)
    
    self.artifactLabel = private.objects.ui.label()
    self.artifactLabel:init(self.frame, "|c00FFFF00" .. L("Artifact") .. "|r", 16)
    self.artifactLabel.frame:SetPoint("TOPLEFT", 265, -12)
    
    self.keystoneLabel = private.objects.ui.label()
    self.keystoneLabel:init(self.frame, "|c00FFFF00" .. L("Keystones") .. "|r", 16)
    self.keystoneLabel.frame:SetPoint("TOPRIGHT", -15, -12)
    
    self.races = {}
    
    -- Set up the race bars
    for i = 1, #private.artifacts do
        self.races[i] = private.objects.ui.raceBar()
        self.races[i]:init(i, self.frame)
        self.races[i]:update() -- Uncomment when ready
    end
    self:ShowAll()
end

--function private.objects.ui.raceFrame:SetNumRaces(num)
--    self.frame:SetHeight(117 + 20 * num)
--end

function private.objects.ui.raceFrame:ShowContinent(num)
    if continents[num] then
        -- Hide them all first
        for i = 1, #self.races do
            self.races[i].frame:Hide()
        end
        for i = 1, #continents[num] do
            self.races[continents[num]].frame:Show()
            self.races[continents[num]].frame:SetPoint("TOP", 0, -15 - (20 * i))
        end
    self.frame:SetHeight(117 + 20 * #continents[num])
    else
        self:ShowAll()
    end
end

function private.objects.ui.raceFrame:ShowAll()
    self.frame:SetHeight(117 + 20 * #private.artifacts)
    for i = 1, #private.artifacts do
        self.races[i].frame:Show()
        self.races[i].frame:SetPoint("TOP", 0, -15 - (20 * i))
    end
end

--test = private.objects.ui.raceFrame()
--test:init(UIParent)
--test:SetNumRaces(9)
--
--test3 = private.objects.ui.raceBar()
--test3:init(4, test.frame)
--test3.frame:SetPoint("TOP", 0, -15)